using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Feel
{
    public class CustomScooterHelmet : MVRScript
    {
        #region Vars  

        private int _initLoops = 0;
        private int _maxInitLoops = 1000;

        protected JSONStorableBool IsShowStrap;

        private Transform ObjectRoot;
        private Transform StrapObj;
        private Transform CoatObj;
        private Transform Cushion1Obj;
        private Transform Cushion2Obj;

        private Material MainMaterial;
        private Material InsideMaterial1;
        private Material InsideMaterial2;
        private Material StrapMaterial;
        private JSONStorableColor MainColor;
        private JSONStorableColor InsideColor;

        #endregion

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("CustomScooter - Please add this script on a Custom Unity Asset ( script disabled).");
                    return;
                }

                _initLoops = 0;

                IsShowStrap = new JSONStorableBool("IsShowStrap", true, OnIsShowStrap);
                RegisterBool(IsShowStrap);
                UIDynamicToggle boostertoggle = CreateToggle(IsShowStrap, false);
                boostertoggle.label = "Strap On Off";

                HSVColor hsvc1 = HSVColorPicker.RGBToHSV(0f, 0.854f, 0.796f);
                MainColor = new JSONStorableColor("Main Color", hsvc1, MainColorCallback);
                RegisterColor(MainColor);
                CreateColorPicker(MainColor, false);

                HSVColor hsvc2 = HSVColorPicker.RGBToHSV(0.627f, 0.396f, 0.105f);
                InsideColor = new JSONStorableColor("Inside Color", hsvc2, InsideColorCallback);
                RegisterColor(InsideColor);
                CreateColorPicker(InsideColor, false);

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Custom Scooter Helmet</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Customize your helmet.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

                var btn = CreateButton("Go To Support", true);
                btn.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        private void OnIsShowStrap(JSONStorableBool jb)
        {
            StrapObj.gameObject.SetActive(jb.val);
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            if (_initLoops < _maxInitLoops)
            {
                _initLoops++;
                FirstInit();
            }
            else
            {
                if (_initLoops == 1001)
                {
                    SuperController.LogError("VamGirlExperience Load Failure " + _initLoops);
                    _initLoops += 1;
                }
            }
        }

        private void FirstInit()
        {
            ObjectRoot = getObjectRoot(containingAtom.reParentObject, "scooter_helmet");

            if (ObjectRoot != null)
            {
                StrapObj = getChildRoot(ObjectRoot, "Strap");
                CoatObj = getChildRoot(ObjectRoot, "Coat");
                Cushion1Obj = getChildRoot(ObjectRoot, "Cushion1");
                Cushion2Obj = getChildRoot(ObjectRoot, "Cushion2");

                Renderer renderer = CoatObj.GetComponent<Renderer>();
                MainMaterial = renderer.materials[0];

                StrapMaterial = StrapObj.GetComponent<Renderer>().materials[0];
                InsideMaterial1 = Cushion1Obj.GetComponent<Renderer>().materials[0];
                InsideMaterial2 = Cushion2Obj.GetComponent<Renderer>().materials[0];

                MainColorCallback(MainColor);
                InsideColorCallback(InsideColor);

                _initLoops += _maxInitLoops + 100;
            }
        }

        void OnDestroy()
        {
            
        }

        private Transform getObjectRoot(Transform parent, string rootName)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(rootName))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child, rootName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private Transform getChildRoot(Transform parent, string searchName)
        {
            foreach (Transform child in parent)
            {
                if (child.name == searchName)
                {

                    return child;
                }
                else
                {
                    Transform childSearch = getChildRoot(child, searchName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private void MainColorCallback(JSONStorableColor dc)
        {
            if (ObjectRoot == null) return;

            Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

            MainMaterial.SetColor("_Color", c);

        }

        private void InsideColorCallback(JSONStorableColor dc)
        {
            if (ObjectRoot == null) return;

            Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

            StrapMaterial.SetColor("_Color", c);
            InsideMaterial1.SetColor("_Color", c);
            InsideMaterial2.SetColor("_Color", c);

        }

    }

}